# -*- coding: utf-8 -*-

"""
PikaNetwork.py
~~~~~~~~~
:license: MIT, see LICENSE for more details.
"""

# Meta.

__title__ = 'pikanetwork.py'
__author__ = 'LetsChill'
__copyright__ = 'LetsChill 2023'
__license__ = 'MIT'

__version__ = '1.0.2'

# structure

from pikanetwork.api import PikaAPI
